from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
import datetime
from datetime import datetime, timedelta
import base64

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def randomkeys(configs, username, args):
    try:
        async with aiofiles.open(configs, "r") as streamr:
            data = yaml.safe_load(await streamr.read())
            if "huntermap" not in data:  
                return
            rand = data.get("huntermap", [])
            if rand is None:
                return
            randomkey = random.choice(data.get("huntermap", []))
                
            if randomkey is None:
                return 
            return randomkey
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at randomkey huntermap: {e}, {type(e)}{reset}", file=sys.stderr)

async def huntermap(domain, session, configs, username, args):
    
    try:
        try:
            
            hunterhows = []
            endtime =  datetime.now().strftime("%Y-%m-%d")
            yeartime =  datetime.now() - timedelta(days=27.8*12)
            inititatetime = yeartime.strftime("%Y-%m-%d")
            query = base64.urlsafe_b64encode(domain.encode("utf-8")).decode('ascii')
            page_size = 100
            page =1 
            
            while True:
                randomapikey = await randomkeys(configs, username, args)
                if not randomapikey:
                    return []
                
                url = f"https://api.hunter.how/search?api-key={randomapikey}&query={query}&start_time={inititatetime}&end_time={endtime}&page={page}&page_size={page_size}"
                proxy = args.proxy if args.proxy else None
                async with session.get(url, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                    
                    data = await response.json()
                    if response.status !=200:
                        if args.show_key_info:
                            if not args.no_color:
                             print(f"[{bold}{red}ALERT{reset}]: Huntermap blocking our request, {username} please check your api usage for this key: {randomapikey}", file=sys.stderr)
                            else:
                                print(f"[ALERT]: Huntermap blocking our request, {username} please check your api usage for this key: {randomapikey}", file=sys.stderr)
                        if len(hunterhows)>0:
                            return hunterhows
                        return []
                    
                    subdomains =  data.get("data", {}).get("list", [])
                    
                    total =  data.get("data", {}).get("total", 0)
                        
                    for subdomain in subdomains: 
                        subdomain =  subdomain['domain']
                        if subdomain.endswith(f"{domain}"):
                            hunterhows.append(subdomain)
                            
                    if total <= len(hunterhows):
                        return hunterhows
                    
                    page +=1
                    
        except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color:  
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Huntermap API, due to Serverside Error", file=sys.stderr)     
                else:
                    print(f"[INFO]: Timeout reached for Huntermap API, due to Serverside Error", file=sys.stderr)   
        except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color:  
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Huntermap API, due to Timeout Error", file=sys.stderr)     
                else:
                    print(f"[INFO]: Timeout reached for Huntermap API, due to Timeout Error", file=sys.stderr)
        except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color:   
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Huntermap API, due to ClientSide connection Error", file=sys.stderr)      
                else:  
                    print(f"[INFO]: Timeout reached for Huntermap API, due to Clientside connection Error", file=sys.stderr)    
        except AttributeError as e:
            return hunterhows
        except KeyboardInterrupt as e:
            quit()
        except Exception as e:
            return hunterhows
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}INFO{reset}]: {bold}{white}At main huntermap: {e}, {type(e)}{reset}", file=sys.stderr)        
                    
                    
            
            


from colorama import Fore, Style
import aiofiles
import random
import yaml
import httpx
import sys
from fake_useragent import UserAgent
import asyncio

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

Trickest = []
sem = asyncio.Semaphore(50)


async def get_count(session, offset, domain, args):
    try:
        response = await session.request("GET", f"""https://api.trickest.io/solutions/v1/public/solution/a7cba1f1-df07-4a5c-876a-953f178996be/view?q=hostname ~ ".{domain}"&dataset_id=a0a49ca9-03bb-45e0-aa9a-ad59082ebdfc&limit=50&offset={offset}&select=hostname&orderby=hostname""", timeout=args.timeout)
        data = response.json()
        count = data.get("total_count")
        if count:
            return count
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api trickest get count block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
            

async def fetcher(session, offset, domain, args):
    try:
        async with sem:
            response = await session.request("GET", f"""https://api.trickest.io/solutions/v1/public/solution/a7cba1f1-df07-4a5c-876a-953f178996be/view?q=hostname ~ ".{domain}"&dataset_id=a0a49ca9-03bb-45e0-aa9a-ad59082ebdfc&limit=50&offset={offset}&select=hostname&orderby=hostname""", timeout=httpx.Timeout(read=300.0, connect=args.timeout, write=None, pool=None))
            data = response.json()
            results = data.get('results', [])
            if results:
                for result in results:
                    subdomain = result['hostname']
                    if subdomain.endswith(f".{domain}"):
                        Trickest.append(subdomain)
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api trickest fetcher block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
            

async def trickest(domain, configs, username, args): # well we can do it in while but will give very slow performance.
    try:
        tasks = []
        try:
            async with aiofiles.open(configs, "r") as streamr:
                data = yaml.safe_load(await streamr.read())
                if "trickest" not in data:
                    return []
                randvalue = data.get("trickest", [])
                if randvalue is None:  
                    return []
            randomkey = random.choice(data.get("trickest", []))
        except yaml.YAMLError as e:
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
        except TypeError as e:
            pass 
        except yaml.YAMLObject as e:
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
        except KeyboardInterrupt:
            exit()
        except asyncio.CancelledError:
            exit()
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in trickest reader block: {e}, {type(e)}{reset}",file=sys.stderr)
        if randomkey is None:
            return []
        try:
            headers = {
                "User-Agent": UserAgent().random,
                "Authorization": f"Token {randomkey}"
            }
            async with httpx.AsyncClient(verify=False, proxy=args.proxy if args.proxy else None, headers=headers) as requests:
                total = await get_count(requests, 10, domain, args)
                if not total or total == 0:
                    return Trickest
                finals = int(total) / 10
                offset = 10
                for _ in range(0,int(finals)+1,1):
                    task = asyncio.ensure_future(fetcher(requests, offset, domain, args))
                    tasks.append(task)
                    offset+=10
                await asyncio.gather(*tasks, return_exceptions=False)
        except httpx._exceptions.ConnectTimeout:
            if args.show_timeout_info:
                if not args.no_color:   
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Trickest API, due to ClientSide connection Error", file=sys.stderr)      
                else:  
                    print(f"[INFO]: Timeout reached for Trickest API, due to Clientside connection Error", file=sys.stderr) 
        except httpx._exceptions.ConnectError:
            if args.show_timeout_info:
                if not args.no_color:   
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Unable to establish client connection to Trickest API{reset}", file=sys.stderr)      
                else:  
                    print(f"[INFO]: Unable to establish client connection to Trickest API", file=sys.stderr) 
        except httpx._exceptions.ReadTimeout:
            pass
        except KeyboardInterrupt:
            exit()
        except asyncio.CancelledError:
            exit()
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]Exception occred in trickest: {e}, {type(e)}", file=sys.stderr)
    except KeyboardInterrupt:
        exit()
    except asyncio.CancelledError:
        exit()
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api trickest main block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
    finally:
        return Trickest


from setuptools import setup, find_packages
with open("README.md", "r", encoding="utf-8") as streamr:
    long_description = streamr.read()

setup(
    name='subdominator',
    version='1.0.9',
    author='D. Sanjai Kumar',
    author_email='bughunterz0047@gmail.com',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/RevoltSecurities/Subdominator",
    description='Subdominator - An ultimate subdomain enumeration tool for Security Researchers and Bug Bounty Hunters',
    packages=find_packages(),
    install_requires=[
        'aiofiles>=23.2.1',
        'aiohttp>=3.9.4',
        'appdirs>=1.4.4',
        'art>=6.1',
        'httpx>=0.27.2',
        'fake_useragent>=1.5.0',
        'beautifulsoup4>=4.11.1',
        'PyYAML>=6.0.1',
        'Requests>=2.31.0',
        'rich>=13.7.1',
        'urllib3>=1.26.18'
    ],
    entry_points={
        'console_scripts': [
            'subdominator = subdominator.subdominator:main'
        ]
    },
)



from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
from fake_useragent import UserAgent
import re

filter = re.compile(r"^(?:\d{1,3}\.){3}\d{1,3}\\032")
bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

Passivetotals = []

async def passivetotal(domain, session, configs, username, args):
    try:
        try:
            async with aiofiles.open(configs, "r") as streamr:
                data = yaml.safe_load(await streamr.read())
                if "passivetotal" not in data:
                    return []
                rand = data.get("passivetotal", [])
                if rand is None:
                    return []
                randomkeys = random.choice(data.get("passivetotal", []))
                if randomkeys is None:
                    return []
                username, randomkey = randomkeys.split(":")
        except yaml.YAMLError as e:
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
        except TypeError as e:
            pass 
        except KeyboardInterrupt as e:
            quit()
        except yaml.YAMLObject as e:
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in passivetotal reader block: {e}, {type(e)}{reset}", file=sys.stderr)
        if username is None or randomkey is None:
            return []
        try:
            url = f"https://api.passivetotal.org/v2/enrichment/subdomains?query={domain}"
            headers = {
                "User-Agent": UserAgent().random,
                "Content-Type": "application/json"
            }
            async with session.get(url, headers=headers, auth=aiohttp.BasicAuth(username, randomkey), ssl=False, timeout=args.timeout, proxy=args.proxy if args.proxy else None) as response:
                jdata = await response.json()
                subdomains = jdata.get("subdomains", [])
                for subdomain in subdomains:
                    if filter.match(subdomain):
                        continue
                    filtered_subdomain = f"{subdomain}.{domain}"
                    Passivetotals.append(filtered_subdomain)
        except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Passive API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Passive API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Passivetotal API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Passivetotal API, due to Clientside connection Error", file=sys.stderr) 
                    
        except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Passivetotal API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Passivetotal API, due to Timeout Error", file=sys.stderr) 
                
        except KeyboardInterrupt as e:
            quit()
            
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api passivetotal req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
                    
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api passivetotal main block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
    finally:
        return Passivetotals


from colorama import Fore,Back,Style
import random
red =  Fore.RED
green = Fore.GREEN
magenta = Fore.MAGENTA
cyan = Fore.CYAN
mixed = Fore.RED + Fore.BLUE
blue = Fore.BLUE
yellow = Fore.YELLOW
white = Fore.WHITE
reset = Style.RESET_ALL
bold = Style.BRIGHT
colors = [ green, cyan, blue]
random_color = random.choice(colors)


def help(path):
    print(f"""
          
{bold}{white}[{reset}{bold}{blue}DESCRIPTION{reset}{bold}{white}]{reset}: {bold}{white}Subdominator a passive subdomain enumeration that discovers subdomains for your targets using with passive and open source resources{reset}

{bold}{white}[{reset}{bold}{blue}USAGE{reset}{bold}{white}]{reset}: {bold}{white}

    subdominator [flags]{reset}
    
{bold}{white}[{reset}{bold}{blue}FLAGS{reset}{bold}{white}]{reset}: {bold}{white}

    {bold}{white}[{reset}{bold}{blue}INPUT{reset}{bold}{white}]{reset}: {bold}{white}
    
        -d,   --domain                  :  domain name to enumerate subdomains.
        -dL,  --domain-list             :  filename that contains domains for subdomain enumeration.
        stdin/stdout                    :  subdominator now supports stdin/stdout
        
    {bold}{white}[{reset}{bold}{blue}OUTPUT{reset}{bold}{white}]{reset}: {bold}{white}
    
        -o,   --output                  :  filename to save the outputs.
        -oD,  --output-directory        :  directory name to save the outputs (use it when -dL is flag used).
        -oJ,  --output-json             :  filename to save output in json fromat
        
    {bold}{white}[{reset}{bold}{blue}OPTIMIZATION{reset}{bold}{white}]{reset}: {bold}{white}
    
        -t,   --timeout                 :  timeout value for every sources requests. 
    
    {bold}{white}[{reset}{bold}{blue}UPDATE{reset}{bold}{white}]{reset}: {bold}{white}
    
        -up,   --update                 :  update subdominator for latest version but yaml source update required manual to not affect your api keys configurations.
        -duc, --disable-update-check    :  disable automatic update check for subdominator
        -sup, --show-updates            :  shows latest version updates of subdominator 
        
    {bold}{white}[{reset}{bold}{blue}CONFIG{reset}{bold}{white}]{reset}: {bold}{white}
    
        -nt,  --notify                  :  send notification of found subdomain using source Slack, Pushbullet, Telegram, Discord
        -p,   --proxy                   :  http proxy to use with subdominator (intended for debugging the performance of subdominator).
        -cp,  --config-path             :  custom path of config file for subdominator to read api keys ( default path: {path})
        -fw,  --filter-wildcards        :  filter the found subdomains with wildcards and give cleaned output
        
    {bold}{white}[{reset}{bold}{blue}DEBUG{reset}{bold}{white}]{reset}: {bold}{white}
    
        -h,   --help                    :  displays this help message and exits 
        -s,   --silent                  :  show only subdomain in output (this is not included for -ski,-sti)
        -v,   --version                 :  show current version of subdominator and latest version if available and exits
        -ski, --show-key-info           :  show keys error for out of credits and key not provided for particular sources
        -ste, --show-timeout-info       :  show timeout error for sources that are timeout to connect
        -nc,  --no-color                :  disable the colorised output of subdominator
        -ls,  --list-source             :  display the sources of subdominator uses for subdomain enumerations and exits (included for upcoming updates on sources).{reset}""")
    
    quit()
    


from colorama import Fore, Style
import aiohttp
import sys
from fake_useragent import UserAgent
import re

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL


async def rapiddns(domain, session, args):
    try:
        
        rapiddnss = []
        
        for pagenum in range(1, 7+1):
            
            url = f"https://rapiddns.io/subdomain/{domain}?page={pagenum}"
            
            auth = {
                "User-Agent": UserAgent().random
            }
            proxy = args.proxy if args.proxy else None
            async with session.get(url, headers=auth, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                
                data = await response.text()
                filterdomain = re.escape(domain)
                pattern = r'(?i)(?:https?://)?([a-zA-Z0-9*_.-]+\.' + filterdomain + r')'
                subdomains = re.findall(pattern, data)
                for subdomain in subdomains:
                    rapiddnss.append(subdomain)
                    
                if "Next" not in data:
                    return rapiddnss
                
    except TimeoutError as e:
        if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Rapiddns, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Rapiddns, due to Timeout Error", file=sys.stderr)
            
    
    except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Rapiddns, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Rapiddns, due to Serverside Error", file=sys.stderr)
            
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Rapiddns, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Rapiddns, due to Clientside connection Error", file=sys.stderr) 
                
    except Exception as e:
        print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at rapiddsn: {e}, {type(e)}{reset}", file=sys.stderr)



from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys
from fake_useragent import UserAgent

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def c99(domain, session, configs, username, args):
    try:
        try:
            
            async with aiofiles.open(configs, "r") as streamr:
                data = yaml.safe_load(await streamr.read())
                if "c99" not in data:
                    return []
                randvalue = data.get("c99", [])
                    
                if randvalue is None:  
                    return []
                       
                randomkey = random.choice(data.get("c99", []))
        except yaml.YAMLError as e:
        
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                           
        except TypeError as e:
            pass 
        except yaml.YAMLObject as e:
        
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                
        except KeyboardInterrupt as e:
            quit()
        
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in c99 reader block: {e}, {type(e)}{reset}",file=sys.stderr)
            
        if randomkey is None:
            return 
        try:
            url = f"https://api.c99.nl/subdomainfinder?key={randomkey}&domain={domain}&json=true"
            proxy = args.proxy if args.proxy else None
            headers = {"User-Agent": UserAgent().random}
            async with session.get(url, headers=headers,timeout=args.timeout, proxy=proxy , ssl=False) as response:
                if response.status != 200:
                    return []
                data = await response.json()
                if "subdomain" in data:
                    C99= [entry["subdomain"] for entry in data]
                    return C99
                    
        except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for C99 API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for C99 API, due to Serverside Error", file=sys.stderr)
                        
        except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for C99 API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for C99 API, due to Clientside connection Error", file=sys.stderr)
        except KeyError as e:
            pass 
        except TypeError as e:
            pass
        except KeyboardInterrupt as e:
            quit()
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api C99 req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api c99 api main block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
    


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def leakix(domain, session, configs, username, args):
    
    try:
        
        try:
            
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "leakix" not in data:
                    
                    return
                
                rand = data.get("leakix", [])
                
                if rand is None:
                    
                    return
                
                randomkey = random.choice(data.get("leakix", []))
                
                if randomkey is None:
                    
                    return
                
        except yaml.YAMLError as e:
            
            if args.show_key_info: 
                
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                      
                else:
                    
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                               
        except TypeError as e:
            
            pass 
        
        except KeyboardInterrupt as e:
            quit()
        
        except yaml.YAMLObject as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
                    
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception at leakix reader block: {e}, {type(e)}{reset}", file=sys.stderr)
                    
        try:
            
            leakixs = []
            
            url = f"https://leakix.net/api/subdomains/{domain}"
            
            auth = {"accept": "application/json", 
                       "api-key": f"{randomkey}"
                   }
            
            proxy = args.proxy if args.proxy else None
            async with session.get(url, headers=auth, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                
                data = await response.json()
                if response.status != 200:
                    
                    if args.show_key_info:
                        
                        if not args.no_color:
                        
                            print(f"[{bold}{red}ALERT{reset}]: LeakIx blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                            
                        else:
                            
                            print(f"[ALERT]: LeakIx blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                    return []
                                
                for subdomain in data:
                    
                    subdomain = subdomain.get("subdomain", {})
                    
                    leakixs.append(subdomain)
                    
                return leakixs
            
        except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for LeakIx API, due to Serverside Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for LeakIx API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                 
                if not args.no_color: 
                    
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for LeakIx API, due to ClientSide connection Error", file=sys.stderr)    
                    
                else:
                    
                    print(f"[INFO]: Timeout reached for LeakIx API, due to Clientside connection Error", file=sys.stderr) 
             
        except TimeoutError as e:
                  
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for LeakIx API, due to Timeout Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for LeakIx API, due to Timeout Error", file=sys.stderr)
                  
        except KeyboardInterrupt as e:
            quit()
            
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api leakix req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
                
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api leakix main block: {e}, due to: {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import aiofiles
import random
import yaml
import aiohttp
import sys

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL

async def randomkeys(configs, username, args):
    try:
            async with aiofiles.open(configs, "r") as streamr:
                
                data = yaml.safe_load(await streamr.read())
                
                if "rsecloud" not in data:
                    
                    return 
                
                randvalue = data.get("rsecloud", [])
                
                if randvalue is None:  
                      
                    return []
                   
            randomkey = random.choice(data.get("rsecloud", []))
            
            return randomkey
            
    except yaml.YAMLError as e:
            
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr) 
    except TypeError as e:
        pass 
        
    except yaml.YAMLObject as e:
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)
            
    except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in rsecloud reader block: {e}, {type(e)}{reset}",file=sys.stderr)
                

async def rsecloud(domain, session, configs, username, args):
            
    try:
        rseclouds = []
        page = 1
        jsons = {
            "domain": f"{domain}"
        }
        while True:
            randomkey = await randomkeys(configs, username, args)
            
            if not randomkey:
                return []
            
            url = f"https://api.rsecloud.com/api/v1/subdomains?page={page}"
            
            headers = {
                
                "Content-Type": "application/json",
                "X-API-Key": f"{randomkey}"
            }
            proxy = args.proxy if args.proxy else None
            async with session.post(url, timeout=args.timeout, proxy=proxy, ssl=False, headers=headers, json=jsons) as response:
                
                if response.status != 200:
                    if args.show_key_info:
                        if not args.no_color:
                             print(f"[{bold}{red}ALERT{reset}]: RseCloud blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                        else:
                                print(f"[ALERT]: RseCloud blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                        if len(rseclouds)>0:
                            return rseclouds
                        return []
                    
                data = await response.json()
                
                subdomains =  data.get("data", {})
                
                total = data.get("total_pages", {})
                                
                for subdomain in subdomains:
                    rseclouds.append(subdomain)
                    
                if total == page:
                    
                    return rseclouds
                
                page +=1
    
    except aiohttp.ServerConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color:  
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for RseClouds API, due to Serverside Error", file=sys.stderr)     
                else:
                    print(f"[INFO]: Timeout reached for RseClouds API, due to Serverside Error", file=sys.stderr)
                    
    except TimeoutError as e:
            if args.show_timeout_info:
                if not args.no_color:  
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for RseClouds API, due to Timeout Error", file=sys.stderr)     
                else:
                    print(f"[INFO]: Timeout reached for RseClouds API, due to Timeout Error", file=sys.stderr)
                
    except aiohttp.ClientConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color:   
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for RseClouds API, due to ClientSide connection Error", file=sys.stderr)      
                else:  
                    print(f"[INFO]: Timeout reached for RseClouds API, due to Clientside connection Error", file=sys.stderr)
                          
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api rsecloud request & main block: {e}, due to: {type(e)}{reset}", file=sys.stderr)


from colorama import Fore, Style
import aiofiles
import random
from fake_useragent.fake import UserAgent
import yaml
import aiohttp
import sys
import json

bold = Style.BRIGHT
blue = Fore.BLUE
red = Fore.RED
white = Fore.WHITE
reset = Style.RESET_ALL
Builtwiths = []

async def builtwith(domain,session, configs, username, args): #not tested but we implemented for experimental
    try:
        try:
            async with aiofiles.open(configs, "r") as streamr:
                data = yaml.safe_load(await streamr.read())
                if "builtwith" not in data:
                    return []
                randvalue = data.get("builtwith", [])
                if randvalue is None:  
                    return []
            randomkey = random.choice(data.get("builtwith", []))
        except yaml.YAMLError as e:
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)  
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)         
        except TypeError as e:
            pass 
        except yaml.YAMLObject as e:
            if args.show_key_info: 
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Hey {username} Please check syntax of {configs}{reset}", file=sys.stderr)    
                else:
                    print(f"[INFO]: Hey {username} Please check syntax of {configs}", file=sys.stderr)   
        except KeyboardInterrupt as e:
            quit()
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception in builtwith reader block: {e}, {type(e)}{reset}",file=sys.stderr)
        if randomkey is None:
            return Builtwiths
        try:
            url = f"https://api.builtwith.com/v21/api.json?KEY={randomkey}&HIDETEXT=yes&HIDEDL=yes&NOLIVE=yes&NOMETA=yes&NOPII=yes&NOATTR=yes&LOOKUP={domain}"
            proxy = args.proxy if args.proxy else None
            headers = {
                "User-Agent": UserAgent().random
            }
            async with session.get(url, headers=headers, timeout=args.timeout, proxy=proxy, ssl=False) as response:
                if response.status != 200:
                    if args.show_key_info:
                        if not args.no_color:
                            print(f"[{bold}{red}ALERT{reset}]: Builtwith blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                        else:
                            print(f"[ALERT]: Builtwith blocking our request, {username} please check your api usage for this key: {randomkey}", file=sys.stderr)
                    return Builtwiths
                jdata = await response.json()
                if isinstance(jdata, dict):
                    results = jdata.get("Results", [])
                    if results:
                        for result in results:
                            for chunk in result.get("Result", {}).get("Paths", []):
                                domain = chunk.get("Domain", "")
                                subdomain = chunk.get("SubDomain", "")
                                if domain and subdomain:
                                    Subdomain = f"{subdomain}.{domain}"
                                    Builtwiths.append(Subdomain)
        except aiohttp.ServerConnectionError as e:
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Builtwith API, due to Serverside Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Builtwith API, due to Serverside Error", file=sys.stderr)
                
        except aiohttp.ClientConnectionError as e:
            
            if args.show_timeout_info:
                if not args.no_color: 
                    print(f"[{bold}{red}INFO{reset}]: {bold}{white}Timeout reached for Builtwith API, due to ClientSide connection Error", file=sys.stderr)    
                else:
                    print(f"[INFO]: Timeout reached for Builtwith API, due to Clientside connection Error", file=sys.stderr) 
        except KeyboardInterrupt as e:
            quit()
        except Exception as e:
            if args.sec_deb:
                print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api builtwith req block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
    except Exception as e:
        if args.sec_deb:
            print(f"[{bold}{red}WRN{reset}]: {bold}{white}Exception occured in api builtwith main block: {e}, due to: {type(e)}{reset}", file=sys.stderr)
    finally:
        return Builtwiths


from colorama import Fore, Style
import sys
def reader(filename, args):
    try:
        with open(filename, "r") as streamr:
            data = streamr.read().splitlines()
        return data
    except FileNotFoundError as e:
        if args.no_color:
            print(f"[WRN]:  {filename} could not found please check the file exists or not", file=sys.stderr)
        else:
            print(f"[{Style.BRIGHT}{Fore.RED}WRN{Style.RESET_ALL}]: {Style.BRIGHT}{Fore.WHITE}{filename} could not found please check the file exists or not{Style.RESET_ALL}", file=sys.stderr)
        quit()
    except Exception as e:
        print(f"Exception in file reader modules: {e}, {type(e)}", file=sys.stderr)